Public Delegate Function MyFxDelegate(ByVal nNumVars As Integer, ByRef fX() As Double, ByRef fParam() As Double) As Double
Public Delegate Function SayFxDelegate() As String

Public Class CPowellSearch1

  Dim m_MyFx As MyFxDelegate

  Protected Function MyFxEx(ByVal nNumVars As Integer, ByRef fX() As Double, ByRef fParam() As Double, _
                  ByRef fSmat(,) As Double, ByVal nII As Integer, _
                  ByVal fAlpha As Double) As Double
    Dim I As Integer
    Dim fXX(nNumVars) As Double

    For I = 0 To nNumVars - 1
      fXX(I) = fX(I) + fAlpha * fSmat(nII, I)
    Next I

    MyFxEx = m_MyFx(nNumVars, fXX, fParam)
  End Function

  Protected Function LinSearch_DirectSearch( _
            ByVal nNumVars As Integer, ByRef fX() As Double, ByRef fParam() As Double, _
            ByRef fAlpha As Double, ByRef fSmat(,) As Double, ByVal nII As Integer, _
            ByVal fInitStep As Double, ByVal fMinStep As Double) As Boolean
    Dim F1, F2 As Double

    F1 = MyFxEx(nNumVars, fX, fParam, fSmat, nII, fAlpha)

    Do
      F2 = MyFxEx(nNumVars, fX, fParam, fSmat, nII, fAlpha + fInitStep)
      If F2 < F1 Then
        F1 = F2
        fAlpha += fInitStep
      Else
        F2 = MyFxEx(nNumVars, fX, fParam, fSmat, nII, fAlpha - fInitStep)
        If F2 < F1 Then
          F1 = F2
          fAlpha -= fInitStep
        Else
          ' reduce search step size
          fInitStep /= 10
        End If
      End If
    Loop Until fInitStep < fMinStep

    Return True

  End Function


  Public Function CalcOptim(ByVal nNumVars As Integer, ByRef fX() As Double, ByRef fParam() As Double, _
                    ByVal fEpsFx As Double, ByVal fEpsStep As Double, ByVal nMaxCycles As Integer, _
                    ByRef nIter As Integer, ByRef sErrorMsg As String, _
                    ByVal MyFx As MyFxDelegate) As Double

    Dim I, J, K, L As Integer
    Dim fAlpha, fSum, F1, F2 As Double
    Dim fX1(nNumVars) As Double, fX2(nNumVars) As Double
    Dim fSmat(nNumVars + 1, nNumVars) As Double

    m_MyFx = MyFx
    F1 = MyFx(nNumVars, fX, fParam)
    ' Build fSmat matrix
    For K = 0 To nNumVars - 1
      fX1(K) = fX(K)
      For L = 0 To nNumVars - 1
        fSmat(K, L) = IIf(K = L, 1, 0)
      Next L
    Next K

    sErrorMsg = ""
    nIter = 1
    J = 1

    Do

      nIter += 1

      ' reset row fSmat(nNumVars+1,:)
      For K = 0 To nNumVars - 1
        fSmat(nNumVars, K) = 0
      Next K

      For I = 0 To nNumVars - 1
        fAlpha = 0
        If Not LinSearch_DirectSearch(nNumVars, fX, fParam, fAlpha, fSmat, I, 0.1, 0.0001) Then
          sErrorMsg = "Linear Search failed"
          Exit Do
        End If
        For K = 0 To nNumVars - 1
          fX(K) += fAlpha * fSmat(I, K)
          fSmat(nNumVars, K) = fSmat(nNumVars, K) + fAlpha * fSmat(I, K)
        Next K
      Next I

      fAlpha = 0
      If Not LinSearch_DirectSearch(nNumVars, fX, fParam, fAlpha, fSmat, nNumVars, 0.1, 0.0001) Then
        sErrorMsg = "Linear Search failed"
        Exit Do
      End If

      For K = 0 To nNumVars - 1
        fX(K) += fAlpha * fSmat(nNumVars, K)
        fX2(K) = fX(K)
      Next K
      F2 = MyFx(nNumVars, fX2, fParam)

      ' test end of iterations criteria
      If Math.Abs(F2 - F1) < fEpsFx Then Exit Do
      fSum = 0
      For K = 0 To nNumVars - 1
        fSum += (fX2(K) - fX1(K)) ^ 2
      Next K
      fSum = Math.Sqrt(fSum)
      If fSum < fEpsStep Then Exit Do
      If J >= nMaxCycles Then Exit Do

      J += 1

      ' rotate data
      For K = 0 To nNumVars - 1
        fX1(K) = fX2(K)
      Next K
      F1 = F2

      ' copy fSmat matrix
      For K = 0 To nNumVars - 1
        For L = 0 To nNumVars - 1
          fSmat(K, L) = fSmat(K + 1, L)
        Next L
      Next K

    Loop

    Return F1

  End Function

End Class
